/*
PAI Version 4.02

PAI is a Question Answering System that gains information from a conversation
conducted in a restricted form of the English language.

Copyright (C) 2002 Claudio Scordino (claudio.scordino@quipo.it)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "command.h"

#define question_code "&%$-question-$%&"
#define command_code "&%$-command-$%&"
#define end_code "&%$-end-$%&"

command:: ~command()
{
	comm* p=head;
	comm* q;
	while (p!=NULL)
	{
		q=p->next;
		delete p;
		p=q;
	}
}

void command:: load(istream& in)
{
	int done=0;
	char string [luninfo];
	in >> string;
	if (strcmp(string,end_code)) 
	{
		
		while (done!=1)
		{
			comm* n;
			n=new comm;
			n->next=head;
			head=n;
			in >> string;
			while (strcmp(string,command_code)) {(n->question)+=string; in>> string;}
			in >> string;
			while (strcmp(string,question_code) && strcmp(string,end_code)) {(n->execute)+=string; in>> string;}
			if (!strcmp(string,end_code)) done=1;
		}
	}
}


void command::add(words quest, words ex)
{
	comm* n;
	n=new comm;
	n->next=head;
	head=n;
	(n->question)=quest;
	(n->execute)=ex;	
}


/* For example:

  ques:		2 + 2 .
  p->question:	*** + *** .
  p->execute:	add .
*/

int command::execute (words ques)
{
    comm* p=head;
    int k=0;
    bool found = false;

    while (p!= NULL && found == false){

        found=false;
        for (int k=0;; k++) {
	    if (ques[k]== NULL && p->question[k]==NULL) {found=true;break;}
            if (ques[k]==NULL || p->question[k]==NULL) {break;}
	    if ( (strcmp(p->question[k],"-any-")) && strcmp(ques[k],(p->question[k]))) {break;}
        }   

        if (found==false) {
           if (p->next==NULL) return 0;
           else p=p->next;
        }
    }

    if (found == true) {
       words to_exec;
       for (int w=0;;w++)
	   if (p->execute[w]!=NULL) to_exec+=p->execute[w];
           else break;
       for (int i=0;; i++) {
          if (ques[i] == NULL) break;
          if (!strcmp (p->question[i],"-any-")) 
	            {to_exec+=ques[i];}
       }

       exec(to_exec);
       return 1;
    } else
       return 0;
}


bool command:: writeanswer(words ques)
{
	comm*p=head;
	bool done = false;
	while (p!=NULL && done==false)
	{
		if ( (p->question) == ques)
		{
			    (p->execute).write();
			    cout << endl;
			done=true;
		}
		p=p->next;
	}
	return done;
}

void command::save (ostream& out)
{
	comm* p=head;
	while (p!=NULL)
	{
		out << question_code << "\n";
		(p->question).save(out);
		out << command_code << "\n";
		(p->execute).save(out);
		p=p->next;
	}
	out << end_code << "\n";
}
