/*
PAI Version 4.02

PAI is a Question Answering System that gains information from a conversation
conducted in a restricted form of the English language.

Copyright (C) 2002 Claudio Scordino (claudio.scordino@quipo.it)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <new.h>

//#define win
#define luninfo 20


#include "verbs.cpp"

#ifdef win
#define commandsfile "commands.ia" //File 4 commands
#define speedanswersfile "spdans.ia" //File 4 speed answer
#define memoryfile  "memory.ia"     //File of memory
#define verbsfile  "verbs.ia"   //File 4 verb table
#endif

#if !defined (win)
#define commandsfile "/usr/local/Pai/commands.ia" //File 4 commands
#define speedanswersfile "/usr/local/Pai/spdans.ia" //File 4 speed answer
#define memoryfile  "/usr/local/Pai/memory.ia"   //File of memory
#define verbsfile  "/usr/local/Pai/verbs.ia"   //File 4 verb table
#endif


char username [luninfo]; //User name
char userobject [luninfo];  //User's
char inputfilename [luninfo]; // Read from a file
verbs verbstable;


#include "words.cpp"

words inputsentence;

#include "memory.cpp"
#include "execute.cpp"
#include "command.cpp"

command commandstable;
command speedanswertable;
memory mem;



#if !defined (win)
void endofmemory ()
{
	cerr <<"End of memory\a\a\a\a" << endl;
	exit (-1);
}
#endif

#include "start.cpp"
#include "reading.cpp"
//#include "talking.cpp"



int main(int argc, char* argv[])
{
#if !defined (win)
	set_new_handler(endofmemory);
#endif

    int firstword=1;
    init();

    // Help
    if (argc==2 && (!strcmp(argv[1],"--help"))) {
        cout <<"PAI - question answering system - version 4.02" << endl;
        cout <<"Copyright (C) 2002 Claudio Scordino" << endl;
        cout << endl;

        cout << "Usage:" << endl;
        cout << endl;
        cout << "pai --help" << endl;
        cout << endl;
        cout << "	Displays this help."<<endl;
        cout << endl;
        cout << endl;

        cout << "pai [--username name] affirmation" << endl;
        cout << endl;
        cout << "	Learns the affirmation and stores it in its internal memory." << endl;
        cout << endl;
        cout << endl;

        cout << "pai [--username name] question" << endl;
        cout << endl;
        cout << "	Tries to answer the question basing on the previous affirmations."<<endl;
        cout << endl;
        cout << endl;

        cout << "pai --readfile filename" << endl;
        cout << endl;
        cout << "	Learns affirmations from a text file."<<endl;
        cout << endl;
        cout << endl;

        cout << "pai --forceanswer sentence -then- answer" << endl;
        cout << endl;
        cout << "	Learns to answer with 'answer' every time the user types 'sentence'."<<endl;
        cout << endl;
        cout << endl;

	cout << "pai --externalapplication sentence -then- command" << endl;
        cout << endl;
        cout << "	Learns to use the external application every time the user types 'sentence'."<<endl;
        cout << "	Using the keyword -any- in 'sentence', PAI will append any word in that position to" << endl;
        cout << "	the application specified with 'command'." << endl;
        cout << endl;
        cout << endl;
        
        cout << "pai --show" << endl;
        cout << endl;
        cout << "	Shows its memory."<<endl;
        cout << endl;
        cout << endl;
        return 0;
        return 0;
    }

    // Show the memory
    if (argc==2 && (!strcmp(argv[1],"--show"))) {
	mem.write();
	return 0;
    }

    // Read a file
    if (argc==3 && (!strcmp(argv[1],"--readfile")) && (strlen(argv[2])<(luninfo)) ) {
        strcpy (inputfilename,argv[2]);
        reading();
    }
    
    // Force special answer
    else if (argc>4 && (!strcmp(argv[1],"--forceanswer"))) {
       words sent;
       words answ;
       int i;
       for (i=2; i<argc; i++)
           if (strlen(argv[i]) > luninfo)
	      return 0;
       
       for (i=2; i<argc; i++){
	   if (strcmp(argv[i],"-then-"))
              sent+=argv[i];
           else break;
       }
       for (i++; i<argc; i++)
           answ+=argv[i];

       speedanswertable.add(sent,answ);
    }

    // Use external application
    else if (argc>4 && (!strcmp(argv[1],"--externalapplication"))) {
       words sent;
       words appl;
       int i;
       for (i=2; i<argc; i++)
           if (strlen(argv[i]) > luninfo)
	      return 0;
       
       for (i=2; i<argc; i++){
	   if (strcmp(argv[i],"-then-"))
              sent+=argv[i];
           else break;
       }
       for (i++; i<argc; i++)
           appl+=argv[i];

       commandstable.add(sent,appl);
    }


    else {
       
       //Initialization of variables
       strcpy(username,"you");
       strcpy(userobject,"your");
	    
       // User name specified
       if (argc>2 && (!strcmp(argv[1],"--username")) && (strlen(argv[2])<(luninfo-2)) ) {
            strcpy (username,argv[2]);
	    strcpy (userobject,username);
	    strcat(userobject,"\'s");
	    firstword+=2;
       }

       // Read data from input
       for (int i=firstword; i<argc; i++)
           if (strlen(argv[i]) < luninfo)
               inputsentence += argv[i];


       //Affirmation
       if (strcmp(argv[argc-1],"?")) {
          if (!(speedanswertable.writeanswer(inputsentence)) && !(commandstable.execute(inputsentence))) {
        // Remove some symbols
       inputsentence -= "?";
       inputsentence -= ".";
       inputsentence -= ";";
       inputsentence -= ",";

            inputsentence.talking_mirror();
             words o,v,a;
             int ty,ti;

             int ret =inputsentence.split4affermation(ty,ti,o,v,a);
             if (ret==1) {
	       /*o.write();
	       cout << endl;
	       v.write();
	       cout << endl;
	       a.write();
	       cout << endl;*/
	       mem.insert(o,v,a,ty,ti);
	     }
         }
      }
      else {// Question
          if (!(speedanswertable.writeanswer(inputsentence)) && !(commandstable.execute(inputsentence))) {
	       // Remove some symbols
       inputsentence -= "?";
       inputsentence -= ".";
       inputsentence -= ";";
       inputsentence -= ",";

     inputsentence-="do";
             inputsentence-="does";
             inputsentence-="did";
             inputsentence-="what";
             inputsentence-="which";
             inputsentence-="where";
             inputsentence-="who";
             inputsentence-="why";
             inputsentence-="how";
             inputsentence-="Do";
             inputsentence-="Does";
             inputsentence-="Did";
             inputsentence-="What";
             inputsentence-="Which";
             inputsentence-="Where";
             inputsentence-="Who";
             inputsentence-="Why";
             inputsentence-="How";
             inputsentence.talking_mirror();
	     int ty,ti;
             words o,v,a;
             int ret=inputsentence.split4interrogation(ty,ti,o,v,a);
             if (ret==1) {
		int given=mem.interrogation(o,v,a);
                if (given==0) given=mem.interrogation(a,v,o);//last resource
                if (given==0) cout <<"I don't know"<< endl;
                else cout << endl;
             }
	     else cout <<"Sorry, I do not understand..."<<endl;
         }

     }
   }
   //cout << "Qui ci siamo 1"<< endl;
    
   fstream ff;

   ff.open(commandsfile,ios::out);
   if (!(!ff)) commandstable.save(ff);
   ff.close();

   //cout << "Qui ci siamo 2"<< endl;
   ff.open(speedanswersfile,ios::out);
   if (!(!ff)) speedanswertable.save(ff);
   ff.close();
   //cout << "Qui ci siamo 3"<< endl;

   ff.open(memoryfile,ios::out);
   if (!(!ff)) mem.save(ff);
   ff.close();
   //cout << "Qui ci siamo 4"<< endl;

   exit(1);
}
