/*
PAI Version 4.02

PAI is a Question Answering System that gains information from a conversation
conducted in a restricted form of the English language.

Copyright (C) 2002 Claudio Scordino (claudio.scordino@quipo.it)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "memory.h"

#define object_code "&%$-object-$%&"
#define verb_code "&%$-verb-$%&"
#define attrib_code "&%$-attrib-$%&"
#define end_code "&%$-end-$%&"


memory:: ~memory()
{
	sentence* p=head;
	sentence* q;
	while (p!=NULL)
	{
		q=p->next;
		delete p;
		p=q;
	}
}


int memory::del (words ob, words vb, words att) //To delete one or more sentences
{
	int done=0;
	sentence* a=head;
	sentence* b=head;
	while (b!=NULL)
	{
		if ( ((b->object)==ob) && ((b->verb)==vb) && ((b->attrib)==att) )
		{
			if (b==head) {head=b->next; delete b; a=head; b=head;continue;} 
			else {a->next=b->next; delete b; b=a->next;continue;}
			done=1;
		}
		if (b!= head) a=a->next;
		b=b->next;
	}
	return done;

}


int memory::check(words ob, words vb, words att)
{
	sentence* p= head;
	while (p!=NULL)
	{
		if ( ((p->object)==ob) && ((p->verb)==vb) && ((p->attrib)==att) ) return 1;
		p=p->next;
	
	}
	return 0;
}


/*	This function checks if the string w1 is the negation of string w2.
	The parameter difference is always a negation (not, does not, etc.)
*/
int memory:: compare (words w1, words w2, words difference)
{

	words empty;

	if ( w1==w2 ) 
		return 0;
	int wi=0;
	words temp;
	int found=0;
	while (found==0 && w1[wi]!=NULL)
	{
		temp=empty;
		//Move wi to the first word of w1 equal to the first word of difference
		while (w1[wi]!=NULL && strcmp(difference[0],w1[wi]))
		{
			temp+=w1[wi];
			wi++;
		}
		if (w1[wi]==NULL) 
			return 0;

		//See if the strings are equal
		found=1;
		int k=0;
		for (;w1[wi+k]!=NULL && difference[k] != NULL ;k++)
		{
			if (strcmp(difference[k],w1[wi+k]))
			{
				found=0;
				break;
			}
		}

		if (found==1)
			while (w1[wi+k]!=NULL)
			{
				temp+=(w1[wi+k]);
			    k++; 
			}
	}	

	return (temp==w2);
}


void memory::insert (words ob, words vb, words att, int ty, int ti ) //Append a new sentence
{

//Delete	
	words ob_temp, vb_temp, att_temp;
	sentence* p= head;
	words no="not";
	words dont="don't";
	words didnt="didn't";
	words doesnt="doesn't";
	words do_not="do"; do_not+="not"; 
	words does_not="does"; does_not+="not";
	words did_not="did"; did_not+="not";

	int ok_delete=0;
	while (p!=NULL)
	{
		int res1=(p->verb==vb);
		int res2=(p->attrib==att);

		//Controls if the words are something like "not..." where "..." is stored in memory
		if ( res1 && res2 && compare(p->object, ob, no)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->object, ob, did_not)) ok_delete=1;

		//Controls if the words are something like "..." where "not..." is stored in memory
		else if ( res1 && res2 && compare(ob, p->object, no)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(ob, p->object, did_not)) ok_delete=1;
		
		if (ok_delete==1)
		{
			del (p->object,p->verb,p->attrib);
			break;
		}


		res1=(p->object==ob);
		res2=(p->attrib==att);

		//Controls if the words are something like "not..." where "..." is stored in memory
		if ( res1 && res2 && compare(p->verb, vb, no)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->verb, vb, did_not)) ok_delete=1;

		//Controls if the words are something like "..." where "not..." is stored in memory
		else if ( res1 && res2 && compare(vb, p->verb, no)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(vb, p->verb, did_not)) ok_delete=1;
		
		if (ok_delete==1)
		{
			del (p->object,p->verb,p->attrib);
			break;
		}
		
		res1=(p->object==ob);
		res2=(p->verb==vb);
		
		//Controls if the words are something like "not..." where "..." is stored in memory
		if ( res1 && res2 && compare(p->attrib, att, no))ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(p->attrib, att, did_not)) ok_delete=1;

		//Controls if the words are something like "..." where "not..." is stored in memory
		else if ( res1 && res2 && compare(att, p->attrib, no)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, dont)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, didnt)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, doesnt)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, do_not)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, does_not)) ok_delete=1;
		else if ( res1 && res2 && compare(att, p->attrib, did_not)) ok_delete=1;
		
		if (ok_delete==1)
		{
			del (p->object,p->verb,p->attrib);
			break;
		}

	
		p=p->next;
	}
	

//Insert

	if(check(ob,vb,att) == 0)
	{
		if (head==NULL)
		{
			head=new sentence;
			head->next=NULL;
			(head->object)=ob;
			(head->verb)=vb;
			(head->attrib)=att;
			(head->type)=ty;
			(head->time)=ti;
		}

		else
		{
			sentence* p=head;
			for (sentence* q=p->next;q!=NULL;q=q->next) p=q;
			p->next=new sentence;
			p=p->next;
			p->next=NULL;
			(p->object)=ob;
			(p->verb)=vb;
			(p->attrib)=att;
			(p->type)=ty;
			(p->time)=ti;
		}
	}
}


void memory::write()
{
	sentence* p=head;
	while (p!=NULL)
	{
		    (p->object).write();
		    cout << "\t";
		    (p->verb).write();
		    cout << " ("<<p->type<<" "<<p->time<<")\t";;
		    (p->attrib).write();
		    cout << endl;
		p=p->next;
	}
}

void memory::load(istream& in)
{
	char w [luninfo];
	int ty,ti;
	in >> w;
	while (strcmp(w,end_code))
	{
		/* w was object_code */
		words w1,w2,w3;
		in >> w;
		while (strcmp(w,verb_code)) {w1+=w; in>> w;}
		in >> ty;
		in >> ti;
		in >> w;
		while (strcmp(w,attrib_code)) {w2+=w; in>> w;}
		in >> w;
		while (strcmp(w,object_code) && strcmp(w,end_code)) {w3+=w; in>> w;}
		(*this).insert(w1,w2,w3,ty,ti);
	}
}


void memory:: save (ostream& out)
{
	sentence* p=head;
	while (p!=NULL)
	{
		out << object_code <<"\n";
		(p->object).save(out);
		out << verb_code << "\n";
		out << (p->type) << "\n";
		out << (p->time) << "\n";
		(p->verb).save(out);
		out << attrib_code << "\n";
		(p->attrib).save(out);
		p=p->next;
	}
	out << end_code <<"\n";
}


int memory::interrogation (words ob,words vb, words ab)
{
	int found=0;
	int found_resp=1;
	words found_object;
	words found_verb;
	words found_attrib;
	sentence* p=this->head;
	while (p!=NULL)
	{
		int resp=0;
		if ( (similar(p->object,ob)==1) || (similar (p->object,ob)==2) ) ++resp;
		if ( (similar(p->verb,vb)==1  ) || (similar (p->verb,vb)==2  ) ) ++resp;
		if ( (similar(p->attrib,ab)==1) || (similar (p->attrib,ab)==2) ) ++resp;
		if (resp>found_resp)
		{
			found_object=(p->object);
			found_verb=(p->verb);
			found_attrib=(p->attrib);
			found_resp=resp;
			found=1;
		}
		p=p->next;
	}
	if (found==1)
	{
		found_object.write();
		cout << " ";
		found_verb.write();
		cout << " ";
		found_attrib.write();
		cout << " ";
	}
	return found;
}
