/* 
PAI Version 4.02

PAI is a Question Answering System that gains information from a conversation
conducted in a restricted form of the English language.

Copyright (C) 2002 Claudio Scordino (claudio.scordino@quipo.it)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "words.h"

words:: ~words()
{
	elem* p=head;
	elem* q;
	while (p!=NULL)
	{
		q=p->next;
		delete p;
		p=q;
	}
}

void words:: write()
{
	elem* p=head;
	while (p!=NULL)
	{
		cout << p->info <<" ";
		p=p->next;
	}
}

void words::save (ostream& out, int endline=1)
{
	elem* p=head;
	while (p!=NULL)
	{
		if (endline==1) out << p->info<<"\n";
		else out << p->info <<" ";
		p=p->next;
	}
}

words& words::operator+= (char* s) //Append a new element
{
	if (head==NULL)
	{
		head=new elem;
		head->next=NULL;
		strcpy(head->info,s);
		return *this;
	}
	
	elem* p=head;
	for (elem* q=p->next;q!=NULL;q=q->next) p=q; 
	p->next=new elem;
	p=p->next;
	p->next=NULL;
	strcpy(p->info,s);
	return *this;
}


void words::join (words w)
{
	elem *p =w.head;
	while (p!=NULL)
	{
		(*this)+=(p->info);
		p=p->next;
	}
}

words& words:: operator=(const words& w)
{
	if (this != &w)
	{
//Delete
		elem* p=head;
		elem* q;
		while (p!=NULL)
		{
			q=p->next;
			delete p;
			p=q;
		}

//Copy
		head=NULL;
		p=w.head;
		while (p!=NULL)
		{
			char* s=p->info;
			(*this)+=s;
			p=p->next;
		}
	}
	return *this;
	
}


words:: words(const words& w)
{
	head=NULL;
	elem* p;	
	p=w.head;
	while (p!=NULL)
	{
		char* s=p->info;
		(*this)+=s;
		p=p->next;
	}
}

words& words::operator-= (char* s)
{
elem* a=head;
elem* b=head;
while (b!=NULL)
{
	if (!strcmp(b->info,s))
	{
		if (b==head) {head=b->next; delete b; a=head; b=head;continue;} 
		else {a->next=b->next; delete b; b=a->next;continue;}
	}
	if (b!= head) a=a->next;
	b=b->next;
}
return *this;

}

int words::change (char* before, char* after) //To change one or more strings
{
	int done=0;
	elem* a=head;
	while (a!=NULL)
	{
		if (!strcmp(a->info,before)) {strcpy(a->info,after); done=1;}
		a=a->next;
	}
	return done;
}

int words::check (char* s) //To check the number of times a string is in the class
{
	int n=0;
	elem* a=head;
	while (a!=NULL)
	{
		if (!strcmp(a->info,s)) n++;
		a=a->next;
	}
	return n;
}


int words::operator==(words w)
{
	elem* p1=head;
	elem* p2=w.head;
	for(;;)
	{
		if (p1==NULL && p2==NULL) return 1;
		if (p1==NULL) return 0;
		if (p2==NULL) return 0;
		if (strcmp(p1->info,p2->info)) return 0;
		p1=p1->next;
		p2=p2->next;
	}
}


int similar (words w1, words w2)
{
	int similar1= (w1==w2);
	words w3=w1;
	words w4=w2;
	w3-="of";
	w3-="to";
	w3-="from";
	w3-="in";
	w3-="with";
	w3-="on";
	w3-="for";
	w3-="since";
	w3-="a";
	w3-="an";
	w3-="the";
	w3-="over";
	
	w4-="of";
	w4-="to";
	w4-="from";
	w4-="in";
	w4-="with";
	w4-="on";
	w4-="for";
	w4-="since";
	w4-="a";
	w4-="an";
	w4-="the";
	w4-="over";

	int similar2=(w3==w4);
	if (similar1==1) return 1;
	if (similar2==1) return 2;
	return 0;
}	
	


int words:: split4affermation (int& ty,int& ti, words& o, words& v, words& a)
{
	int type,time;
	elem* p= head;
	int position=0;
	int done=0;
	if ((p==NULL)||(p->next==NULL)||((p->next)->next==NULL)) return 0;
	if (verbstable.check((p->info),type,time) !=0) return 0;
	while (1) 
	{
		if (p==NULL) return done;
		int found=verbstable.check((p->info),type,time);
		if (found!=0 && position==0 && (time!=4 || done!=0))
		{
			v+=(p->info);
			ty=type;
			if (done==0) ti=time;
			done = 1;
		}
		else if (done==0) o+=(p->info);
		else {a+=(p->info); position=1;}
		p=p->next;
	}
}

int words:: split4interrogation (int& ty,int& ti, words& o, words& v, words& a)
{
	int type,time;
	elem* p= head;
	int done=0;
	if ((p==NULL)||(p->next==NULL)) return 0;
	while (1) 
	{
		if (p==NULL) return 1;
		int found=verbstable.check((p->info),type,time);  
		if (found!=0)
		{
			v+=(p->info);
			ty=type;
			if (done==0) ti=time;
			done = 1;
		}
		else if (done==0) o+=(p->info);
		else a+=(p->info);
		p=p->next;
	}
}

int words::talking_mirror()
{
	elem* p=this->head;
	while (1) 
	{
		if (p==NULL) return 0;
		if (!strcmp(p->info,"i") || !strcmp(p->info,"I")) 
		{
			strcpy(p->info,username);
			if (p->next==NULL) return 0;
			if (!strcmp(p->next->info,"have")) 
			{
				p=p->next;
				strcpy(p->info,"has");
			}				
				
		}
		// Sentence referred to the user (i.e. I am):
		else if ((!strcmp(p->info,"am")) || (!strcmp(p->info,"Am")) ) {
			if (!strcmp(username,"you"))
				//Username non provided:
				strcpy(p->info,"are");
			else
				//Username provided:
				strcpy(p->info,"is");
		}
		// Sentence referred to the system (i.e. you are):
		else if (!strcmp(p->info,"you") || !strcmp(p->info,"You")) 
		{
			strcpy(p->info,"i");
			if (p->next==NULL) return 0;
			if (!strcmp(p->next->info,"are")) 
			{
				p=p->next;
				strcpy(p->info,"am");
			}
			else if (!strcmp(p->next->info,"were")) 
			{
				p=p->next;
				strcpy(p->info,"was");
			}
		}
		else if ((!strcmp(p->info,"my")) || (!strcmp(p->info,"My"))) strcpy(p->info,userobject);
		else if ((!strcmp(p->info,"mine")) || (!strcmp(p->info,"Mine"))) strcpy(p->info,userobject);
		else if ((!strcmp(p->info,"me")) || (!strcmp(p->info,"Me"))) strcpy(p->info,username);
		else if ((!strcmp(p->info,"your")) || (!strcmp(p->info,"Your"))) strcpy(p->info,"my");
		p=p->next;
	}
}


int words::reading_mirror()
{
	elem* p=this->head;
	while (1) 
	{
		if (p==NULL) return 0;
		if (!strcmp(p->info,"you") || !strcmp(p->info,"You")) 
		{
			strcpy(p->info,"i");
			if (p->next==NULL) return 0;
			if (!strcmp(p->next->info,"are")) 
			{
				p=p->next;
				strcpy(p->info,"am");
			}
			else if (!strcmp(p->next->info,"were")) 
			{
				p=p->next;
				strcpy(p->info,"was");
			}
		}
		else if ((!strcmp(p->info,"your"))  || (!strcmp(p->info,"Your")) ) strcpy(p->info,"my");
		p=p->next;
	}
}



/* Use in this way: 
	for (int k=0;k<10;k++) if (w[k]!=NULL) cout << k <<": "<<(w[k])<<endl;*/
char* words::operator[] (int i)
{
	elem* p=head;
	while (p!=NULL)
	{
		if (i==0) return (p->info);
		i--;
		p=p->next;
	}
	return NULL;
}
